(** @author: Benoît Montagu <benoit.montagu@inria.fr>
    Copyright © Inria 2021
*)

(** Command-line interface for the evaluator *)

open Cmdliner

let interactive_t =
  let open Arg in
  value & flag
  & info [ "interactive" ]
      ~doc:
        "Whether the evaluation must wait for user interaction after each \
         evaluation step. This flag automatically enables tracing."

let trace_t =
  let open Arg in
  value & flag
  & info [ "trace" ]
      ~doc:"Whether the evaluator should print a trace of the execution step."

let tailcalls_t =
  let open Arg in
  value & flag
  & info [ "tailcalls" ]
      ~doc:
        "Whether the evaluator should optimize calls that are performed in a \
         tail position, by not consuming an extra frame on the call stack."

let main_t =
  let open Arg in
  value & opt string "main"
  & info [ "main" ]
      ~doc:
        "Name of the main function of the program, i.e. the function that must \
         be called at the beginning. This function must expect exactly no \
         arguments. By default the function called \"main\" is selected."

let options_t : Main.options Term.t =
  let open Term in
  const (fun interactive trace tailcalls main ->
      Main.{ interactive; trace; tailcalls; main })
  $ interactive_t $ trace_t $ tailcalls_t $ main_t

let file_t =
  let open Arg in
  required
  & pos 0 (some string) None
  & info [] ~docv:"FILE" ~doc:"File to evaluate"

let eval_t =
  let open Term in
  const Main.eval $ options_t $ file_t

let eval_info =
  let open Cmd in
  info "eval"
    ~doc:
      "Small-step evaluator for a simple 'while' language equipped with \
       references and function calls, with programs written in administrative \
       normal form (ANF)."

let () = exit @@ Cmd.(eval @@ v eval_info eval_t)
